/*
 * gsampler-uimanager.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gi18n-lib.h>

#include "gsampler-uimanager.h"
#include "gsampler-utils.h"

G_DEFINE_TYPE (GsamplerUiManager, gsampler_ui_manager, GTK_TYPE_UI_MANAGER);

static GsamplerUiManager* gsampler_ui_manager__instance = NULL;

#if GTK_CHECK_VERSION (2,90,0)
#define KEY_ICON "keyboard.svg"
#define CONSOLE_ICON "console.svg"
#define CARD_ICON "card.svg"
#define SOUND_ICON "sound.svg"
#else
#define KEY_ICON "keyboard.png"
#define CONSOLE_ICON "console.png"
#define CARD_ICON "card.png"
#define SOUND_ICON "sound.png"
#endif

#define UI_DATA "<ui>\n" \
"  <menubar name=\"menubar\">\n" \
"    <menu name=\"FileMenu\" action=\"File\">\n" \
"      <menuitem name=\"FileNewMenu\" action=\"FileNew\"/>\n" \
"      <menuitem name=\"FileOpenMenu\" action=\"FileOpen\"/>\n" \
"      <separator/>\n" \
"      <menuitem name=\"FileSaveMenu\" action=\"FileSave\"/>\n" \
"      <menuitem name=\"FileSaveAsMenu\" action=\"FileSaveAs\"/>\n" \
"      <separator/>\n" \
"      <menuitem name=\"FileQuitMenu\" action=\"FileQuit\"/>\n" \
"    </menu>\n" \
"    <menu name=\"EditMenu\" action=\"Edit\">\n" \
"      <menuitem name=\"EditPreferencesMenu\" action=\"EditPreferences\"/>\n" \
"    </menu>\n" \
"    <menu name=\"ViewMenu\" action=\"View\">\n" \
"      <menuitem name=\"ViewToolbarMenu\" action=\"ViewToolbar\"/>\n" \
"      <menuitem name=\"ViewStatusbarMenu\" action=\"ViewStatusbar\"/>\n" \
"      <separator/>\n" \
"      <menuitem name=\"ViewKeyboardMenu\" action=\"ViewKeyboard\"/>\n" \
"      <menuitem name=\"ViewConsoleMenu\" action=\"ViewConsole\"/>\n" \
"    </menu>\n" \
"    <menu name=\"ServeurMenu\" action=\"Serveur\">\n" \
"      <menuitem name=\"ServeurResetMenu\" action=\"ServeurReset\"/>\n" \
"      <menuitem name=\"ServeurConnectMenu\" action=\"ServeurConnect\"/>\n" \
"    </menu>\n" \
"    <menu name=\"HelpMenu\" action=\"Help\">\n" \
"      <menuitem name=\"StartAssistantMenu\" action=\"StartAssistant\"/>\n" \
"      <separator/>\n" \
"      <menuitem name=\"HelpAboutMenu\" action=\"HelpAbout\"/>\n" \
"    </menu>\n" \
"  </menubar>\n" \
"  <toolbar name=\"toolbar\">\n" \
"    <toolitem action=\"FileNew\" />\n" \
"    <toolitem action=\"FileOpen\" />\n" \
"    <toolitem action=\"FileSave\" />\n" \
"    <separator/>\n" \
"    <toolitem action=\"EditPreferences\" />\n" \
"    <separator/>\n" \
"    <toolitem action=\"ViewKeyboard\" />\n" \
"    <toolitem action=\"ViewConsole\" />\n" \
"    <separator/>\n" \
"    <toolitem action=\"ServeurReset\" />\n" \
"    <toolitem action=\"ServeurConnect\" />\n" \
"  </toolbar>\n" \
"</ui>"

static void
gsampler_ui_manager_add_stockid (void)
{
	GdkPixbuf* pixbuf = NULL;
	GtkIconSet* iset = NULL;
	GtkIconFactory* fact = NULL;
	gchar *load_file = NULL;
	GError * error = NULL;

	fact = gtk_icon_factory_new ();
	GtkStockItem item1 = {.stock_id = GSAMPLER_STOCK_KEYBOARD, .label = "keyboard", .modifier = 0, .keyval = 0, .translation_domain = "keyboard"};
	GtkStockItem item2 = {.stock_id = GSAMPLER_STOCK_CONSOLE, .label = "console", .modifier = 0, .keyval = 0, .translation_domain = "console"};
	GtkStockItem item3 = {.stock_id = GSAMPLER_STOCK_CARD, .label = "card", .modifier = 0, .keyval = 0, .translation_domain = "card"};
	GtkStockItem item4 = {.stock_id = GSAMPLER_STOCK_SOUND, .label = "sound", .modifier = 0, .keyval = 0, .translation_domain = "sound"};
	GtkStockItem items[4] = {item1, item2, item3, item4};
	gtk_stock_add (items, 4);
	gchar *load_files[4] = {KEY_ICON, CONSOLE_ICON, CARD_ICON, SOUND_ICON};
	gint i;
	for (i=0; i<4; i++)
	{
		load_file = gsampler_utils_get_data_file (load_files[i]);
		if (load_file != NULL)
		{
			pixbuf = gdk_pixbuf_new_from_file_at_scale (load_file, -1, -1, TRUE, &error);
			if (error != NULL) 
			{
				g_object_unref (pixbuf);
				g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
				g_error_free (error);
			} else 
			{
				iset = gtk_icon_set_new_from_pixbuf (pixbuf);
				gchar *stock_id = NULL;
				if (string_contains (load_files[i], "keyboard"))
				    stock_id = GSAMPLER_STOCK_KEYBOARD;
				else if (string_contains (load_files[i], "console"))
				    stock_id = GSAMPLER_STOCK_CONSOLE;
				else if (string_contains (load_files[i], "card"))
				    stock_id = GSAMPLER_STOCK_CARD;
				else if (string_contains (load_files[i], "sound"))
				    stock_id = GSAMPLER_STOCK_SOUND;
				if (stock_id != NULL)
					gtk_icon_factory_add (fact, stock_id, iset);
				gtk_icon_set_unref (iset);
				g_object_unref (pixbuf);
			}
			g_free (load_file);
		}
	}
	gtk_icon_factory_add_default (fact);
	g_object_unref (fact);
}

static GObject * 
gsampler_ui_manager_constructor (GType type, guint n_construct_properties, 
                                 GObjectConstructParam * construct_properties) 
{
	GObject * obj;
	GsamplerUiManager * self = NULL;
	GtkActionGroup *group_base= NULL;
	GtkActionGroup *group_client= NULL;
	GtkAction* action= NULL;
	GError * error = NULL;

	obj = G_OBJECT_CLASS (gsampler_ui_manager_parent_class)->constructor (type, n_construct_properties, construct_properties);
	self = GSAMPLER_UI_MANAGER (obj);
	
	gsampler_ui_manager_add_stockid ();	
	gtk_ui_manager_add_ui_from_string ((GtkUIManager*) self, UI_DATA, strlen (UI_DATA), &error);
	if (error != NULL) 
	{
		g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
		g_error_free (error);
		return NULL;
	}
	group_base = gtk_action_group_new ("general");
	group_client = gtk_action_group_new ("client");
	
	action = gtk_action_new ("File", _("_File"), NULL, NULL);
	gtk_action_group_add_action (group_base, action);
	g_object_unref (action);
	action = gtk_action_new ("FileNew", NULL, _("Create an empty session"), GTK_STOCK_NEW);
	gtk_action_group_add_action_with_accel (group_client, action, "<control>N");
	g_object_unref (action);
	action = gtk_action_new ("FileOpen", NULL, _("Open a lscp file"), GTK_STOCK_OPEN);
	gtk_action_group_add_action_with_accel (group_client, action, "<control>O");
	g_object_unref (action);
	action = gtk_action_new ("FileSave", NULL, _("Save a lscp file"), GTK_STOCK_SAVE);
	gtk_action_group_add_action_with_accel (group_client, action, "<ctrl>S");
	g_object_unref (action);
	action = gtk_action_new ("FileSaveAs", NULL, _("Save as a lscp file"), GTK_STOCK_SAVE_AS);
	gtk_action_group_add_action_with_accel (group_client, action, "<ctrl><shift>S");
	g_object_unref (action);
	action = gtk_action_new ("FileQuit", NULL, _("Quit the application"), GTK_STOCK_QUIT);
	gtk_action_group_add_action_with_accel (group_base, action, "<control>Q");
	g_object_unref (action);
	action = gtk_action_new ("Edit", _("_Edit"), NULL, NULL);
	gtk_action_group_add_action_with_accel (group_base, action, NULL);
	g_object_unref (action);
	action = gtk_action_new ("EditPreferences", _("Pr_eferences"),
	                         _("Configure the application"), GTK_STOCK_PREFERENCES);
	gtk_action_group_add_action_with_accel (group_base, action, NULL);
	g_object_unref (action);
	action = gtk_action_new ("Help", _("_Help"), NULL, NULL);
	gtk_action_group_add_action (group_base, action);
	g_object_unref (action);
	action = gtk_action_new ("StartAssistant", _("Assistant"), _("Start the first start Assistant"), NULL);
	gtk_action_group_add_action (group_base, action);
	g_object_unref (action);
	action = gtk_action_new ("HelpAbout", NULL, _("About this application"), GTK_STOCK_ABOUT);
	gtk_action_group_add_action (group_base, action);
	g_object_unref (action);
	
	action = gtk_action_new ("View", _("_View"), NULL, NULL);
	gtk_action_group_add_action (group_base, action);
	g_object_unref (action);
	action = (GtkAction*) gtk_toggle_action_new ("ViewToolbar", _("Toolbar"),
	                                             _("View/hide the toolbar"), NULL);
	gtk_action_group_add_action_with_accel (group_base, action, NULL);
	g_object_unref (action);
	action = (GtkAction*) gtk_toggle_action_new ("ViewStatusbar", _("Statusbar"),
	                                             _("View/hide the statusbar"), NULL);
	gtk_action_group_add_action_with_accel (group_base, action, NULL);
	g_object_unref (action);
	action = (GtkAction*) gtk_toggle_action_new ("ViewKeyboard", _("Keyboard"),
	                                			 _("View/hide the virtual keyboard"), GSAMPLER_STOCK_KEYBOARD);
	gtk_action_group_add_action_with_accel (group_base, action, NULL);
	g_object_unref (action);
	action = (GtkAction*) gtk_toggle_action_new ("ViewConsole", _("Console"), 
	                                             _("View/hide the console window"), GSAMPLER_STOCK_CONSOLE);
	gtk_action_group_add_action_with_accel (group_base, action, NULL);
	g_object_unref (action);
	
	action = gtk_action_new ("Serveur", _("_Server"), NULL, NULL);
	gtk_action_group_add_action (group_base, action);
	g_object_unref (action);
	action = gtk_action_new ("ServeurReset", _("Reset"),
	                         _("Reset the Serveur/Client"), GTK_STOCK_REFRESH);
	gtk_action_group_add_action_with_accel (group_base, action, NULL);
	g_object_unref (action);
	action = gtk_action_new ("ServeurConnect", _("Restart"),
	                         _("Restart Linuxsampler"), GTK_STOCK_PROPERTIES);
	gtk_action_group_add_action_with_accel (group_base, action, NULL);
	g_object_unref (action);
	
	gtk_ui_manager_insert_action_group ((GtkUIManager*) self, group_base, 0);
	gtk_ui_manager_insert_action_group ((GtkUIManager*) self, group_client, 1);
	
	return obj;
}

static void 
gsampler_ui_manager_init (GsamplerUiManager * self) 
{

}

static void 
gsampler_ui_manager_finalize (GObject* obj) 
{
	G_OBJECT_CLASS (gsampler_ui_manager_parent_class)->finalize (obj);
}

static void 
gsampler_ui_manager_class_init (GsamplerUiManagerClass * klass) 
{
	gsampler_ui_manager_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = gsampler_ui_manager_finalize;
	G_OBJECT_CLASS (klass)->constructor = gsampler_ui_manager_constructor;
}

GsamplerUiManager* 
gsampler_ui_manager_new (void) 
{
	GsamplerUiManager * ui = NULL;
	ui = (GsamplerUiManager*) g_object_new (GSAMPLER_TYPE_UI_MANAGER, NULL);
	return ui;
}

GsamplerUiManager* 
gsampler_ui_manager_get_instance (void) 
{
	GsamplerUiManager* result = NULL;
	if (gsampler_ui_manager__instance == NULL)
	{
		gsampler_ui_manager__instance = gsampler_ui_manager_new ();;
	}
	result = g_object_ref (gsampler_ui_manager__instance);
	return result;
}

GtkAction* 
gsampler_ui_manager_get_action (GsamplerUiManager* self, const gchar* name) 
{
	GList *action_groups = NULL;
	
	action_groups = gtk_ui_manager_get_action_groups ((GtkUIManager*) self);
	for (; action_groups; action_groups = action_groups->next) 
	{
		GtkActionGroup *action_group = (GtkActionGroup *) action_groups->data;
		GtkAction *action = gtk_action_group_get_action (action_group, name);
		if (action != NULL)
			return action;
	}
	return NULL;
}

void 
gsampler_ui_manager_action_set_sensitive (GsamplerUiManager* self, 
                                          const gchar* name, gboolean sensi) 
{
	GtkAction* action;
	g_return_if_fail (self != NULL);
	g_return_if_fail (name != NULL);

	action = gsampler_ui_manager_get_action (self, name);
	if (action != NULL) {
		gtk_action_set_sensitive (action, sensi);
	} else 
	{
		g_warning ("Can't get the action %s for set it sensitive", name);
	}
}

void 
gsampler_ui_manager_action_set_toggled (GsamplerUiManager* self,
                                        const gchar* name, gboolean active) 
{
	GtkToggleAction* action;
	g_return_if_fail (self != NULL);
	g_return_if_fail (name != NULL);

	action = (GtkToggleAction*) gsampler_ui_manager_get_action (self, name);
	if (action != NULL) 
	{
		gtk_toggle_action_set_active (action, active);
	} else 
	{
		g_warning ("Can't get the toggleaction %s for set it active", name);
	}
}

void
gsampler_ui_manager_group_set_sensitive (GsamplerUiManager* self,
                                         const gchar* name, gboolean sensi)
{
	GList *action_groups = NULL;
	
	action_groups = gtk_ui_manager_get_action_groups ((GtkUIManager*) self);
	for (; action_groups; action_groups = action_groups->next) 
	{
		GtkActionGroup *action_group = (GtkActionGroup *) action_groups->data;
		gchar* group_name = (gchar*) gtk_action_group_get_name (action_group);
		if (g_strcmp0 (group_name, name) == 0) 
		{
			gtk_action_group_set_sensitive (action_group, sensi);
		}
	}

}
